#!/sbin/ksh


#########################################################################
#
#
#  MIT License:
#
#
#  Copyright (c) 2021 TruHobbyist
#
#  Permission is hereby granted, free of charge, to any person obtaining
#  a copy of this software and associated documentation files (the
#  "Software"), to deal in the Software without restriction, including
#  without limitation the rights to use, copy, modify, merge, publish,
#  distribute, sublicense, and/or sell copies of the Software, and to
#  permit persons to whom the Software is furnished to do so, subject to
#  the following conditions:
#
#  The above copyright notice and this permission notice shall be
#  included in all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
#  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
#  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
#  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
#  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#  OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
#  WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
#
#  GPL2:
#
#
#  File Looper v1.2.1 - Loopback device driver for SGI IRIX
#  Copyright (C) 2021 TruHobbyist
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Library General Public
#  License version 2 as published by the Free Software Foundation.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Library General Public License version 2 for more details.
#
#  You should have received a copy of the GNU Library General Public
#  License version 2 along with this source code; if not, write to the
#  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
#  Boston, MA  02110-1301, USA.
#
#
#########################################################################


# NOTE: Template
#
#  install.sh.template has the following context:
#
#   - OS
#   - CPU bitness
#   - Product name
#   - Product version


# NOTE: OS
#
#  For IRIX, the CPU bitness is appended to the OS name:
#
#   IRIX32 for 32 bit systems
#   IRIX64 for 64 bit systems


OS="IRIX32"
DEV_FILE="/dev/fl"


# Product parameters
PRODUCT_NAME='fl'
PRODUCT_NAME_FULL='file looper'
PRODUCT_NAME_FULL_CAP='File looper'
PRODUCT_VERSION="1.2.1"


# Operation to be performed
# OP=0 : INSTALL
# OP=1 : UNINSTALL
OP=


# System manpage root
#MAN_ROOT=/usr/share/man
MAN_ROOT=/usr/share/catman


# Debug mode:
# DBG=0 : Execute command
# DBG=1 : Do _not_ execute command (dry run)
DBG=0


# Verbose
VERBOSE=1


# TTY
TTY=`tty`


# System commands
TEST="          /sbin/test"
BASENAME="      /sbin/basename"
ECHO="          /sbin/echo"
TRUE="          /usr/bin/true"
RM="            /sbin/rm"
MKDIR="         /sbin/mkdir"
RMDIR="         /usr/bin/rmdir"
CP="            /sbin/cp"
HEAD="          /usr/bsd/head"
CUT="           /usr/bin/cut"
ID="            /usr/bin/id"
WC="            /sbin/wc"
LS="            /sbin/ls"
TR="            /usr/bin/tr"
GREP="          /sbin/grep"
MORE="          /usr/bsd/more"
LN="            /sbin/ln"
CAT="           /sbin/cat"
EXPR="          /sbin/expr"
TAIL="          /usr/bin/tail"
DATE="          /sbin/date"
MV="            /sbin/mv"
SED="           /sbin/sed"
AWK="           /usr/bin/awk"
SORT="          /usr/bin/sort"
COMM="		/usr/bin/comm"
READ=read
UNIQ="          /usr/bin/uniq"
MKNOD="         /sbin/mknod"
ML="            /sbin/ml"
AUTOCONFIG="    /etc/autoconfig"


CMD_NAMES="$BASENAME $TRUE $RM $MKDIR $RMDIR $CP $HEAD $CUT $ID $WC $LS $TR $GREP $SYSCONFIG $MORE $LN $CAT $EXPR $TAIL $DATE $MV $SED $AWK"


exec_cmd()
{
        RET=
        CMD=${1}
        EXIT_ON_ERROR=${2}
        ERROR_MSG=${3}
        RETVAL=0


        # Check for command
        if ${TEST} "${CMD:-x}" = x
        then
                ${ECHO} "ERROR: No command given to exec_cmd()" > $TTY
                exit 1
        fi


        # Check for verbose switch
        if ${TEST} $VERBOSE -eq 1
        then
                ${ECHO} $CMD > $TTY
        fi


        # Check EXIT_ON_ERROR flag
        if ${TEST} "$EXIT_ON_ERROR" = 'EXIT_ON_ERROR'
        then
                EXIT_ON_ERROR=1
        else
                EXIT_ON_ERROR=0
        fi


        # Execute if not debugging
        if ${TEST} $DBG -eq 0
        then
                RET=`eval $CMD`
                RETVAL=$?


                # Check for command output
                if ${TEST} "$RET" != ''
                then
                        ${ECHO} "$RET"
                fi


                # Check return value
                if ${TEST} $RETVAL -eq 1
                then
                        # Check for error message
                        if ${TEST} -n "$ERROR_MSG"
                        then
                                ${ECHO} $ERROR_MSG > $TTY
                        fi


                        # Check for exit flag
                        if ${TEST} $EXIT_ON_ERROR -eq 1
                        then
                                exit 1
                        fi
                fi
        fi


        return $RETVAL
}


mod_unload()
{
        MOD_ID=$1


        # Unload kernel module
        ${ML} unld $MOD_ID > /dev/null 2>&1


        # Check for unsuccessful unload
        if ${TEST} $? != 0
        then    
                VECHO "WARNING: Couldn't unload kernel module $PRODUCT_NAME"
        else    
                VECHO "Module unloaded"
        fi
}


mod_unreg()
{       
        MOD_ID=$1

        
        # Unregister kernel module
        ${ML} unreg $MOD_ID > /dev/null 2>&1

        
        # Check for unsuccessful unregistration
        if ${TEST} $? != 0
        then    
                VECHO "WARNING: Couldn't unregister kernel module $PRODUCT_NAME"
        else    
                VECHO "Module unregistered"
        fi
}


get_mod_id()
{       
        MOD=$1

        
        # Get module name
        MOD_ID=-1
        MOD=`${ECHO} $MOD | ${AWK} '{print $2 " " $7}' | ${SED} 's/,$//' | ${TR} -d '\n'`
        MOD_NAME=`${ECHO} $MOD | ${CUT} -d ' ' -f 2`

        
        # Check for fl_ module prefix
        if ${TEST} "$MOD_NAME" = "${PRODUCT_NAME}_"
        then    
                # NOTE: fl kernel module is registered and loaded

                
                # Get module ID 
                MOD_ID=`${ECHO} $MOD | ${CUT} -d ' ' -f 1`
        fi

        
        ${ECHO} $MOD_ID
}


mod_is_loaded()
{       
        # Check for prefix 
        MOD=`${ML} list -b | ${GREP} ${PRODUCT_NAME}_ 2>&1`

        
        MOD_ID=-1
        if ${TEST} $? -eq 0
        then    
                MOD_ID=`get_mod_id "$MOD"`
        fi

        
        ${ECHO} $MOD_ID
}


mod_is_reg()
{       
        # Check for prefix 
        MOD=`${ML} list -r | ${GREP} ${PRODUCT_NAME}_ 2>&1`

        
        MOD_ID=-1
        if ${TEST} $? -eq 0
        then    
                MOD_ID=`get_mod_id "$MOD"`
        fi

        
        ${ECHO} $MOD_ID
}


VECHO()
{       
        MSG=$1

        
        # Check for verbose switch
        if ${TEST} $VERBOSE -eq 1
        then    
                ${ECHO} $MSG
        fi
}


print_syntax()
{
        ${ECHO} "\nSyntax:\n"
        ${ECHO} "  ./install.sh [-v] [-i | -u]"
        ${ECHO} "  ./install.sh -h\n"
        ${ECHO} "  -v verbose"
        ${ECHO} "  -i install product"
        ${ECHO} "  -u uninstall product"
        ${ECHO} "  -h print this help menu\n"
        ${ECHO} "When no command line switches are specified, a quiet product install is performed."
}


check_args()
{
        # Number of arguments without script name
        NUM_ARGS=${#}


        # Syntax:
        # install.sh [-v] [-i | -u]
        # install.sh -h

        if [ $NUM_ARGS -gt 2 ]
        then
                ${ECHO} "ERROR: Too many arguments"
                print_syntax
                exit 1
        fi


        ARG1=${1}
        ARG2=${2}


        # Switch through number of arguments
        case $NUM_ARGS in

                0)
                        VERBOSE=0
                        OP=0
                        ;;

                1)
                        # ./install.sh -v
                        # ./install.sh -i
                        # ./install.sh -u
                        # ./install.sh -h


			# NOTE: /sbin/test bug
			#
			#  In IRIX, /sbin/test does not evaluate correctly a string argument
			#  when it is preceded by a hyphen and is a valid flag for a test primitive.
			#
			#  The expected behaviour would be to evaluate such a string as a string literal,
			#  and not as a flag.
			#
			#  A workaround is to use a case ... in construct.


			# Check first arg
			case $ARG1 in
				'-v')
					VERBOSE=1
					OP=0
					;;
				'-i')
					VERBOSE=0
					OP=0
					;;
				'-u')
					VERBOSE=0
					OP=1
					;;
				'-h')
					print_syntax
					exit 1
					;;
				*)
                                	${ECHO} "ERROR: Unknown command line switch: $ARG1"
                                	print_syntax
                                	exit 1
					;;
			esac

                        ;;

                2)
                        # ./install.sh -v -i
                        # ./install.sh -v -u


                        if ${TEST} "$ARG1" != '-v'
                        then
                                ${ECHO} "ERROR: Expected -v but got: $ARG1"
                                print_syntax
                                exit 1
                        fi


                        VERBOSE=1


			# Check second arg
			case $ARG2 in

				'-i')
					OP=0
					;;
				'-u')
					OP=1
					;;
				*)
                                	${ECHO} "ERROR: Unknown operation: $ARG2"
                                	print_syntax
                                	exit 1
					;;
			esac


                        ;;

                *)
                        ;;
        esac


        return 0
}


install_op()
{
        ${ECHO} "INFO: Installing $PRODUCT_NAME_FULL version $PRODUCT_VERSION on $OS"
        ${ECHO} Checking...


        # Check if product already exists on the system
	TGT=/usr/opt/$PRODUCT_NAME/$PRODUCT_VERSION
	if ${TEST} -d $TGT
	then
                ${ECHO} "The directory $TGT already exists on this system. This directory is required to install the file looper product."


                while ${TRUE}
                do
                        ${ECHO} "Do you want to delete $TGT? [y]n " | ${TR} -d '\n'
                        read yn


                        # Check for answer
                        if ${TEST} "$yn" = 'y' -o "$yn" = ''
                        then
                                ${ECHO} "Deleting $TGT"
				exec_cmd "${RM} -rf $TGT" EXIT_ON_ERROR "ERROR: Couldn't remove $TGT. Please try to remove it manually and rerun this script. Thank you!"

                                break
                        elif ${TEST} "$yn" = 'n';
                        then
                                exit 0
                        fi
                done
        fi


        # NOTE: /usr/opt/$PRODUCT_NAME/$PRODUCT_VERSION does not exist


        ${ECHO} Installing...


	# Check for device file /dev/fl
        if ${TEST} -c $DEV_FILE
        then
                ${RM} $DEV_FILE
        fi


	# Set package root
	PACKAGE_ROOT="./usr/opt/$PRODUCT_NAME/$PRODUCT_VERSION"


        # Check if /dev/loop exists on the system
        TGT=/dev/loop
        if ${TEST} -d $TGT
        then
                ${ECHO} "The loop directory $TGT already exists on this system. This directory is required to create the loop devices."


                while ${TRUE}
                do
                        ${ECHO} "Do you want to delete $TGT? [y]n " | ${TR} -d '\n'
                        read yn


                        # Check for answer
                        if ${TEST} "$yn" = 'y' -o "$yn" = ''
                        then
                                ${ECHO} "Deleting $TGT"
                                exec_cmd "${RM} -rf $TGT" EXIT_ON_ERROR "ERROR: Couldn't remove $TGT. Please try to remove it manually and rerun this script. Thank you!"

                                break
                        elif ${TEST} "$yn" = 'n';
                        then
                                exit 0
                        fi
                done
        fi


        # NOTE: /dev/loop does not exist


        # Create /dev/loop
        TGT="/dev/loop"
        exec_cmd "${MKDIR} $TGT" EXIT_ON_ERROR "ERROR: Couldn't create $TGT. Please report back this error. Thank you!"


	# NOTE: /dev/loop created


        # Check for /usr/opt/fl
	TGT=/usr/opt/$PRODUCT_NAME
        if ${TEST} ! -d $TGT
	then
                # Create product directory
                exec_cmd "${MKDIR} -p $TGT" EXIT_ON_ERROR "ERROR: Couldn't create $TGT. Please try to create it manually and rerun this script. Thank you!"
        fi


	# NOTE: Copy /usr/opt/fl/1.2.1 to /usr/opt/fl


        # Copy version subdir to /usr/opt/fl/1.2.1
	SRC="./usr/opt/$PRODUCT_NAME/$PRODUCT_VERSION"
	DST="/usr/opt/$PRODUCT_NAME"
        exec_cmd "${CP} -r $SRC $DST" EXIT_ON_ERROR "ERROR: Couldn't copy $SRC to $DST. Please try to copy it over manually and rerun this script. Thank you!"


	# Copy daemon to /usr/sbin
	SRC="$PACKAGE_ROOT/usr/sbin/fld"
	DST="/usr/sbin"
        exec_cmd "${CP} $SRC $DST" EXIT_ON_ERROR "ERROR: Couldn't copy $SRC to $DST. Please try to copy it over manually and rerun this script. Thank you!"


	# Copy daemon init script to /etc/init.d
	SRC="$PACKAGE_ROOT/etc/init.d/fld"
	DST="/etc/init.d"
        exec_cmd "${CP} $SRC $DST" EXIT_ON_ERROR "ERROR: Couldn't copy $SRC to $DST. Please try to copy it over manually and rerun this script. Thank you!"


        # Check for symbolic link /etc/rc2.d/S99fld
	TGT=/etc/rc2.d/S99fld
	CREATE_SYMLINK=0
        if ${TEST} -l $TGT
	then
        	exec_cmd "${RM} $TGT" CONT_ON_ERROR "WARNING: Couldn't remove symbolic link $TGT. Skipped."


		if ${TEST} $? -eq 0
		then
			# NOTE: Old symbolic link removed


			CREATE_SYMLINK=1
		fi
	else
		# NOTE: Symbolic link not found


		CREATE_SYMLINK=1
	fi


	# Check for symbolic link creation
	if ${TEST} $CREATE_SYMLINK -eq 1
	then
		# Create symbolic link to init script
		SRC="/etc/rc2.d/S99fld"
		DST="../init.d/fld"
		exec_cmd "${LN} -s $DST $SRC" EXIT_ON_ERROR "ERROR: Couldn't link $SRC to $DST. Please try to create the symbolic link manually and rerun this script. Thank you!"
	fi


        # Copy manpage
	SRC=$PACKAGE_ROOT/catman/cat1/$PRODUCT_NAME.z
	DST=$MAN_ROOT/u_man/cat1
        exec_cmd "${CP} $SRC $DST" EXIT_ON_ERROR "ERROR: Couldn't copy $SRC to $DST. Please try to copy it over manually and rerun this script. Thank you!"


	# NOTE: Copy mount files to system


        # Copy ./usr/bin/mount_iso9660
        SRC=$PACKAGE_ROOT/usr/bin/mount_iso9660
        DST=/usr/bin
	exec_cmd "${CP} $SRC $DST" EXIT_ON_ERROR "ERROR: Couldn't copy $SRC to $DST. Please try to copy it over manually and rerun this script. Thank you!"


        # Copy ./usr/lib32/mount_iso9660.so
        SRC=$PACKAGE_ROOT/usr/lib32/mount_iso9660.so
        DST=/usr/lib32
	exec_cmd "${CP} $SRC $DST" EXIT_ON_ERROR "ERROR: Couldn't copy $SRC to $DST. Please try to copy it over manually and rerun this script. Thank you!"


	# NOTE: Copy sysgen files to system


        # Copy ./var/sysgen/boot/$PRODUCT_NAME.o
        SRC=$PACKAGE_ROOT/var/sysgen/boot/$PRODUCT_NAME.o
        DST=/var/sysgen/boot 
	exec_cmd "${CP} $SRC $DST" EXIT_ON_ERROR "ERROR: Couldn't copy $SRC to $DST. Please try to copy it over manually and rerun this script. Thank you!"


        # Copy ./var/sysgen/master.d/$PRODUCT_NAME
        SRC=$PACKAGE_ROOT/var/sysgen/master.d/$PRODUCT_NAME
        DST=/var/sysgen/master.d
	exec_cmd "${CP} $SRC $DST" EXIT_ON_ERROR "ERROR: Couldn't copy $SRC to $DST. Please try to copy it over manually and rerun this script. Thank you!"


        # Copy ./var/sysgen/system/$PRODUCT_NAME.sm
        SRC=$PACKAGE_ROOT/var/sysgen/system/$PRODUCT_NAME.sm
        DST=/var/sysgen/system/$PRODUCT_NAME.sm
	exec_cmd "${CP} $SRC $DST" EXIT_ON_ERROR "ERROR: Couldn't copy $SRC to $DST. Please try to copy it over manually and rerun this script. Thank you!"


        # Copy ./var/sysgen/mtune/$PRODUCT_NAME
        SRC=$PACKAGE_ROOT/var/sysgen/mtune/$PRODUCT_NAME
        DST=/var/sysgen/mtune/$PRODUCT_NAME
	exec_cmd "${CP} $SRC $DST" EXIT_ON_ERROR "ERROR: Couldn't copy $SRC to $DST. Please try to copy it over manually and rerun this script. Thank you!"


	# NOTE: Merge stune file


	# Check for /var/sysgen/stune file
	if ${TEST} ! -f "/var/sysgen/stune"
	then
		# NOTE: /var/sysgen/stune does not exist


                ${ECHO} "ERROR: The file /var/sysgen/stune does not exist on this system. This file is required to install the file looper product."


		exit 1
	fi


	# Backup /var/sysgen/stune
	SRC=/var/sysgen/stune
	DST=$PACKAGE_ROOT/var/sysgen/stune.bak
	exec_cmd "${CP} $SRC $DST" EXIT_ON_ERROR "ERROR: Couldn't backup $SRC. Please try to copy it over manually and rerun this script. Thank you!"


	# Sort /var/sysgen/stune
        SRC=/var/sysgen/stune
        DST=/var/sysgen/stune.sorted
	exec_cmd "${SORT} $SRC > $DST" EXIT_ON_ERROR "ERROR: Couldn't sort $SRC. Please report back this error. Thank you!"


        # Sort ./var/sysgen/$PRODUCT_NAME.stune
        SRC=$PACKAGE_ROOT/var/sysgen/$PRODUCT_NAME.stune
        DST=$PACKAGE_ROOT/var/sysgen/$PRODUCT_NAME.stune.sorted
	exec_cmd "${SORT} $SRC > $DST" EXIT_ON_ERROR "ERROR: Couldn't sort $SRC. Please report back this error. Thank you!"


	# Check for file looper systune variables
        SRC=/var/sysgen/stune.sorted
        DST=$PACKAGE_ROOT/var/sysgen/$PRODUCT_NAME.stune.sorted
	exec_cmd "${COMM} -12 $SRC $DST > $DST.comm" EXIT_ON_ERROR "ERROR: Couldn't check for common systune variables. Please report back this error. Thank you!"


	if ${TEST} "`${WC} -l $DST.comm | awk '{print $1}'`" != "0"
	then
		${ECHO} "ERROR: Found file looper systune variables in system stune file. Please remove the following lines from /var/sysgen/stune and rerun this script. Thank you!"


		exec_cmd "${CAT} $DST.comm" EXIT_ON_ERROR "ERROR: Couldn't display common systune variables. Please report back this error. Thank you!"


		exit 1
	fi


	# Add file looper's systune variables
	SRC=$PACKAGE_ROOT/var/sysgen/$PRODUCT_NAME.stune
	DST=/var/sysgen/stune
	exec_cmd "${CAT} $SRC >> $DST" CONT_ON_ERROR "ERROR: Couldn't add lines in $SRC to $DST. Please try to add these lines manually. Thank you!"


	# Check for error
	if ${TEST} $? -ne 0
	then
		# NOTE: Error
		#
		#  Couldn't add lines in $SRC to $DST.


		# Restore $PACKAGE_ROOT/var/sysgen/stune.bak
		SRC=$PACKAGE_ROOT/var/sysgen/stune.bak
		DST=/var/sysgen/stune
		exec_cmd "${CP} $SRC $DST" EXIT_ON_ERROR "ERROR: Couldn't restore backup $SRC to $DST. Please try to restore this file manually. Thank you!"
	fi


	# Remove /var/sysgen/stune.sorted
	SRC=/var/sysgen/stune.sorted
	exec_cmd "${RM} $SRC" CONT_ON_ERROR "INFO: Couldn't delete $SRC. Please try to delete this file manually. Skipped."


	# Autoconfigure system
	${AUTOCONFIG}


	${ECHO} "\n\n$PRODUCT_NAME_FULL_CAP version $PRODUCT_VERSION has been successfully installed.\n"
	${ECHO} "Adjust your PATH by adding /usr/opt/$PRODUCT_NAME/$PRODUCT_VERSION/bin in order to use the \`$PRODUCT_NAME' user command.\n\n"
	${ECHO} "Manpages are installed in system default path $MAN_ROOT and should be immediately available."
	${ECHO} ""
	${ECHO} "You may start by just typing \`$PRODUCT_NAME -h' to invoke the help menu or read the full documentation by typing \`man $PRODUCT_NAME'."
	${ECHO} ""
	${ECHO} "Thank you for choosing our product!\n"


	return 0
}


uninstall_op()
{
        ${ECHO} "INFO: Uninstalling $PRODUCT_NAME_FULL version $PRODUCT_VERSION on $OS"
        ${ECHO} Checking...


	# Check if fl product already exists on the system
        if ${TEST} ! -d /usr/opt/$PRODUCT_NAME/$PRODUCT_VERSION
        then
                ${ECHO} "ERROR: $PRODUCT_NAME_FULL_CAP version $PRODUCT_VERSION product not installed on this system."
                exit 1
        fi


        # NOTE: Kernel modules
        #
        #  In IRIX, file looper can be either:
        #
        #   - Registered
        #   - Registered and loaded
        #
        #  In both cases, the module should be unloaded and/or un-
        #  registered.


        # Check for status of fl kernel module
        VECHO "Checking for loaded module"
        MOD_ID=`mod_is_loaded`

        
        # Check for registered and loaded module
        if ${TEST} $MOD_ID -ne -1
        then
                VECHO "Unloading module"

        
                # Unload module
                mod_unload $MOD_ID
        else
		# NOTE: Check for registered module


                MOD_ID=`mod_is_reg`
        fi
        
        
        # Check for registered module
        VECHO "Checking for registered module"
        if ${TEST} $MOD_ID -ne -1
        then
                VECHO "Unregistering module"
        

                # Unregister module
                mod_unreg $MOD_ID
        fi

        
        # NOTE: Kernel module unloaded and/or unregistered


	${ECHO} Uninstalling...


        # NOTE: /var/sysgen/stune
        #
        #  The system tunables file may have been modified with
        #  new system tunable variables since installation of
        #  this product.
        #
        #  A simple replacement of the backup file in the products
        #  package directory is therefore not possible, as it could
        #  overwrite newly added systune variables.
        #
        #  File looper's system tunable variables have to be filtered
        #  out one by one.


        PRODUCT_ROOT="/usr/opt/$PRODUCT_NAME/$PRODUCT_VERSION"


        # Copy /var/sysgen/stune
        SRC=/var/sysgen/stune
        DST=/var/sysgen/stune.new
        exec_cmd "${CP} $SRC $DST" EXIT_ON_ERROR "ERROR: Couldn't copy $SRC to $DST. Please try to copy it over manually and rerun this script. Thank you!"


        # Remove systune variables
        FILTER_ERROR=0
        SRC=$PRODUCT_ROOT/var/sysgen/$PRODUCT_NAME.stune
        DST=/var/sysgen/stune
        while ${READ} line
        do
                line=`${ECHO} "$line" | ${AWK} '{print $1}'`
                exec_cmd "${GREP} -v \"$line\" $DST.new > $DST.part" NO_EXIT_ON_ERROR


                if ${TEST} $? -ne 0
                then
                        FILTER_ERROR=1


                        break
                fi


                # Rename .part to .new
                PART=$DST.part
                NEW=$DST.new
                exec_cmd "${MV} $PART $NEW" EXIT_ON_ERROR "ERROR: Couldn't rename $PART to $NEW. Please try to rename it manually and rerun this script. Thank you!"
        done < $SRC
        if ${TEST} $FILTER_ERROR -eq 0
        then
                if ${TEST} -f $DST.new
                then
                        # Create new /var/sysgen/stune
                        exec_cmd "${MV} $DST.new $DST" NO_EXIT_ON_ERROR


                        if ${TEST} $? -ne 0
                        then
                                FILTER_ERROR=1
                        fi
                fi
        fi


        # Check for filter error
        if ${TEST} $FILTER_ERROR -eq 1
        then
                ${ECHO} "ERROR: Couldn't filter out file looper systune variables. Please remove manually the following systune variables from $DST. Thank you!"


                ${CAT} $SRC
        fi


        # Delete /var/sysgen/mtune/fl
        TGT=/var/sysgen/mtune/$PRODUCT_NAME
        exec_cmd "${RM} $TGT > /dev/null 2>&1" NO_EXIT_ON_ERROR


	# Delete /dev/fl
	TGT=$DEV_FILE
	exec_cmd "${RM} $TGT > /dev/null 2>&1" NO_EXIT_ON_ERROR


	# Remove /dev/loop
        TGT="/dev/loop"
        exec_cmd "${RM} -rf $TGT > /dev/null 2>&1" NO_EXIT_ON_ERROR


        # Remove product version directory
	TGT="/usr/opt/$PRODUCT_NAME/$PRODUCT_VERSION"
        exec_cmd "${RM} -rf $TGT > /dev/null 2>&1" NO_EXIT_ON_ERROR
        

        # Check for empty product directory
	TGT="/usr/opt/$PRODUCT_NAME"
        RET=`exec_cmd "${LS} $TGT | ${WC} -l" NO_EXIT_ON_ERROR`
        

        if ${TEST} $RET -eq 0
        then
                # Remove product directory
		TGT="/usr/opt/$PRODUCT_NAME"
                exec_cmd "${RMDIR} $TGT > /dev/null 2>&1" NO_EXIT_ON_ERROR
        fi


	# Delete daemon
	TGT=/usr/sbin/fld
	exec_cmd "${RM} $TGT > /dev/null 2>&1" NO_EXIT_ON_ERROR


	# Delete daemon init script
	TGT=/etc/init.d/fld
	exec_cmd "${RM} $TGT > /dev/null 2>&1" NO_EXIT_ON_ERROR


	# Delete symbolic link to daemon init script
	TGT=/etc/rc2.d/S99fld
	exec_cmd "${RM} $TGT > /dev/null 2>&1" NO_EXIT_ON_ERROR


	# Delete manpage
	TGT=$MAN_ROOT/u_man/cat1/$PRODUCT_NAME.z
	exec_cmd "${RM} $TGT > /dev/null 2>&1" NO_EXIT_ON_ERROR


	# Delete mount_iso9660
	TGT=/usr/bin/mount_iso9660
	exec_cmd "${RM} $TGT > /dev/null 2>&1" NO_EXIT_ON_ERROR


	# Delete mount_iso9660.so
	TGT=/usr/lib32/mount_iso9660.so
	exec_cmd "${RM} $TGT > /dev/null 2>&1" NO_EXIT_ON_ERROR


	# Delete /var/sysgen/boot/fl.o
	TGT=/var/sysgen/boot/$PRODUCT_NAME.o
	exec_cmd "${RM} $TGT > /dev/null 2>&1" NO_EXIT_ON_ERROR


	# Delete /var/sysgen/master.d/fl
	TGT=/var/sysgen/master.d/$PRODUCT_NAME
	exec_cmd "${RM} $TGT > /dev/null 2>&1" NO_EXIT_ON_ERROR


	# Delete /var/sysgen/system/fl.sm
	TGT=/var/sysgen/system/$PRODUCT_NAME.sm
	exec_cmd "${RM} $TGT > /dev/null 2>&1" NO_EXIT_ON_ERROR


	# TODO: Remove systune variables


	# Autoconfigure system
	${AUTOCONFIG}


        ${ECHO} "\n\nFile looper version $PRODUCT_VERSION has been successfully uninstalled.\n"
        ${ECHO} "Adjust your PATH by removing /usr/opt/$PRODUCT_NAME/$PRODUCT_VERSION/bin"
        ${ECHO} ""
        ${ECHO} "Thank you for having used our product!\n"


        return 0
}


# Check for echo command
if ${ECHO} echo > /dev/zero 2>&1;
then
        NOOP=
else
        exit 1
fi


# Check for test command
if  ${TEST} "x" > /dev/zero 2>&1;
then
        NOOP=
else
        ${ECHO} "ERROR: Couldn't find test command. This command is necessary for the execution of this script. Please add it to the system and rerun ./install.sh. Thank you!"
        exit 1
fi


# Check for commands
for CMD in $CMD_NAMES
do
        if ${TEST} ! -x $CMD
        then
                ${ECHO} "ERROR: $CMD not found. This command is necessary for the execution of this script. Please add it to the system and rerun ./install.sh. Thank you!"
                exit 1
        fi
done


# NOTE: Required commands are available


# Check if invoking user is root
EUID=`${ID} -u`

if ${TEST} $EUID -ne 0
then
        ${ECHO} "ERROR: Must be superuser to run this script"
        exit 0
fi


# Check arguments
check_args $@


# Switch
if ${TEST} $OP = 0
then

        install_op

elif ${TEST} $OP = 1
then

        uninstall_op

fi


